//
// C++ Implementation: mediumpluginchooser
//
// Description:
//
//
// Author: Jeff Mitchell <kde-dev@emailgoeshere.com>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "amarok.h"
#include "debug.h"
#include "devicemanager.h"
#include "mediabrowser.h"
#include "medium.h"
#include "mediumpluginchooser.h"
#include "plugin/pluginconfig.h"
#include "pluginmanager.h"
#include "transferdialog.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qvbox.h>

#include <kapplication.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <klineedit.h>
#include <klocale.h>
#include <kpushbutton.h>

TransferDialog::TransferDialog( MediaDevice * mdev )
        : KDialogBase( amaroK::mainWindow(), "transferdialog", true, QString::null, Ok|Cancel, Ok )
{
    m_accepted = false;
    m_sort1LastIndex = m_sort2LastIndex = -1;

    kapp->setTopWidget( this );
    setCaption( kapp->makeStdCaption( i18n( "Transfer Queue to Device" ) ) );

    QVBox* vbox = makeVBoxMainWidget();
    vbox->setSpacing( KDialog::spacingHint() );

    QString transferDir = mdev->getTransferDir();

    /*new QLabel( i18n( "Enter the directory to use as the transfer operation's root directory.\n"
                                       "%1 will be prepended automatically.\n"
                                       "If the directory does not exist, it will be created.\n" )
                    .arg( transferDir ), vbox );

    KLineEdit *directoryBox = NULL;
    directoryBox = new KLineEdit( QString::null, vbox, "transferdir_lineedit" );
    */

    new QLabel( i18n( "Your music will be transferred to\n%1" )
                    .arg( transferDir ), vbox );


    new QLabel( i18n( "You can have your music automatically grouped in\n"
                      "a variety of ways. Each grouping will create\n"
                      "directories based upon the specified criteria.\n"), vbox );

    m_label1 = new QLabel( i18n( "Select first grouping:\n" ), vbox );
    m_sort1 = new KComboBox( vbox );
    m_label2 = new QLabel( i18n( "Select second grouping:\n" ), vbox );
    m_sort2 = new KComboBox( vbox );
    m_label3 = new QLabel( i18n( "Select third grouping:\n" ), vbox ); 
    m_sort3 = new KComboBox( vbox );

    m_label2->setDisabled(true);
    m_sort2->setDisabled(true);
    m_label3->setDisabled(true);
    m_sort3->setDisabled(true);

    m_combolist = new QPtrList<KComboBox>();
    m_combolist->append( m_sort1 );
    m_combolist->append( m_sort2 );
    m_combolist->append( m_sort3 );

    KComboBox * comboTemp;
    for( comboTemp = m_combolist->first(); comboTemp; comboTemp = m_combolist->next() )
    {
        comboTemp->insertItem( "None" );
        comboTemp->insertItem( "Artist" );
        comboTemp->insertItem( "Album" );
        comboTemp->insertItem( "Genre" );
        comboTemp->setCurrentItem( 0 );
    }

    connect( m_sort1, SIGNAL(activated(int)), SLOT(sort1_activated(int)));
    connect( m_sort2, SIGNAL(activated(int)), SLOT(sort2_activated(int)));

}

void
TransferDialog::slotOk()
{
    m_accepted = true;
    KDialogBase::slotOk();
}

void
TransferDialog::slotCancel()
{
    m_accepted = false;
    KDialogBase::slotCancel();
}

void
TransferDialog::sort1_activated( int index )
{
    //sort3
    if( m_sort2LastIndex > 0 )
        m_sort3->insertItem( m_sort2->text( m_sort2LastIndex ), m_sort2LastIndex );
    if( m_sort1LastIndex > 0 )
        m_sort3->insertItem( m_sort1->text( m_sort1LastIndex ), m_sort1LastIndex );
    if( index > 0 )
        m_sort3->removeItem( index );
    m_sort3->setCurrentItem( 0 );
    m_sort3->setDisabled( true );
    //sort2
    if( m_sort1LastIndex > 0 )
        m_sort2->insertItem( m_sort1->text( m_sort1LastIndex ), m_sort1LastIndex );
    if( index > 0 )
        m_sort2->removeItem( index );
    m_sort2->setCurrentItem( 0 );
    if( index == 0 )
        m_sort2->setDisabled( true );
    else
        m_sort2->setDisabled( false );

    m_sort2LastIndex = 0;
    m_sort1LastIndex = index;
}

void
TransferDialog::sort2_activated( int index )
{
    //sort3
    if( m_sort2LastIndex > 0 )
        m_sort3->insertItem( m_sort2->text( m_sort2LastIndex ), m_sort2LastIndex );
    if( index > 0 )
        m_sort3->removeItem( index );
    m_sort3->setCurrentItem( 0 );
    if( index == 0 )
        m_sort3->setDisabled( true );
    else
        m_sort3->setDisabled( false );

    m_sort2LastIndex = index;
}

#include "transferdialog.moc"
